==> ruleofthumb_2.4.3_2.11-0.0.3-sources.jar: ai/salmonbrain/ruleofthumb/WelchStatisticsTransformer.scala <==
package ai.salmonbrain.ruleofthumb

import org.apache.spark.ml.util.Identifiable
import org.apache.spark.sql.expressions.UserDefinedFunction
import org.apache.spark.sql.functions._
import org.apache.spark.sql.{ DataFrame, Dataset, Row }

/**
 * Transformer to apply Welch test
 * @see https://en.wikipedia.org/wiki/Welch%27s_t-test
 * @param uid -  uid for transformer
 */
class WelchStatisticsTransformer(override val uid: String) extends BaseStatisticTransformer {
  def this() = this(Identifiable.randomUID("welchStatisticsTransformer"))

  override def transform(dataset: Dataset[_]): DataFrame = {
    import dataset.sqlContext.implicits._
    checkVariants(dataset)
    dataset
      .groupBy(
        $(experimentColumn),
        $(metricNameColumn),
        $(additiveColumn),
        $(metricSourceColumn)
      )
      .pivot($(variantColumn))
      .agg(
        struct(
          avg(col($(valueColumn))) as "mean",
          variance(col($(valueColumn))) as "variance",
          count(col($(valueColumn))) as "length"
        )
      )
      .withColumn("statisticsData", doStatistic($(alpha), $(beta))($"control", $"treatment"))
      .drop("control", "treatment")
  }

  def doStatistic(alpha: Double, beta: Double): UserDefinedFunction = udf {
    (
        control: Row,
        treatment: Row
    ) =>
      val statResult = WelchTTest.welchTTest(control, treatment, alpha, beta)
      val controlSize = control.getAs[Long]("length")
      val treatmentSize = treatment.getAs[Long]("length")

      StatisticsReport(
        statResult,
        alpha,
        beta,
        srm(controlSize.toInt, treatmentSize.toInt, $(srmAlpha)),
        controlSize,
        treatmentSize,
        TestType.WELCH.toString
      )
  }
}

==> ruleofthumb_3.1.2_2.12-0.0.5-sources.jar: ai/salmonbrain/ruleofthumb/WelchStatisticsTransformer.scala <==
package ai.salmonbrain.ruleofthumb

import org.apache.spark.ml.util.Identifiable
import org.apache.spark.sql.expressions.UserDefinedFunction
import org.apache.spark.sql.functions._
import org.apache.spark.sql.{ DataFrame, Dataset, Row }

/**
 * Transformer to apply Welch test
 * @see https://en.wikipedia.org/wiki/Welch%27s_t-test
 * @param uid -  uid for transformer
 */
class WelchStatisticsTransformer(override val uid: String) extends BaseStatisticTransformer {
  def this() = this(Identifiable.randomUID("welchStatisticsTransformer"))

  override def transform(dataset: Dataset[_]): DataFrame = {
    import dataset.sqlContext.implicits._
    checkVariants(dataset)
    dataset
      .groupBy(
        $(experimentColumn),
        $(metricNameColumn),
        $(additiveColumn),
        $(metricSourceColumn),
        $(entityCategoryNameColumn),
        $(entityCategoryValueColumn)
      )
      .pivot($(variantColumn))
      .agg(
        struct(
          avg(col($(valueColumn))) as "mean",
          variance(col($(valueColumn))) as "variance",
          count(col($(valueColumn))) as "length"
        )
      )
      .withColumn("statisticsData", doStatistic($(alpha), $(beta))($"control", $"treatment"))
      .drop("control", "treatment")
  }

  def doStatistic(alpha: Double, beta: Double): UserDefinedFunction = udf {
    (
        control: Row,
        treatment: Row
    ) =>
      val statResult = WelchTTest.welchTTest(control, treatment, alpha, beta)
      val controlSize = control.getAs[Long]("length")
      val treatmentSize = treatment.getAs[Long]("length")

      StatisticsReport(
        statResult,
        alpha,
        beta,
        srm(controlSize.toInt, treatmentSize.toInt, $(srmAlpha)),
        controlSize,
        treatmentSize,
        TestType.WELCH.toString
      )
  }
}
